/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#+
#+     Internet Radio Automation & Encoding Toolkit
#+
#+     Copyright (C) 2018, 2019, 2020, 2021, 2022, 2023 
#+     by Kevin C. O'Kane
#+
#+     Kevin C. O'Kane
#+     kc.okane@gmail.com
#+     https://www.cs.uni.edu/~okane
#+     https://threadsafebooks.com/
#+
#+ This program is free software; you can redistribute it and/or modify
#+ it under the terms of the GNU General Public License as published by
#+ the Free Software Foundation; either version 2 of the License, or
#+ (at your option) any later version.
#+
#+ This program is distributed in the hope that it will be useful,
#+ but WITHOUT ANY WARRANTY; without even the implied warranty of
#+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#+ GNU General Public License for more details.
#+
#+ You should have received a copy of the GNU General Public License
#+ along with this program; if not, write to the Free Software
#+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#+
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

//	Nov 2, 2023

#include "includes.h"
#include "defines.h"
#include "common-externs.h"
#include "common-fcn-headers.h"


#include <vlc/libvlc.h>
#include <vlc/libvlc_renderer_discoverer.h>
#include <vlc/libvlc_media.h>
#include <vlc/libvlc_media_player.h>

#define SLASH '/'

void	on_sharpen_clicked(GtkWidget *w) {

	char tmp[16000], filename[4096], filetype[256];
	time_t tmark = time(0);
	int i, j, k, pcount = 0;

	close_boxes();

	GtkAdjustment *adj = 
		gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON(sharp_amount));
	gdouble luma =  gtk_adjustment_get_value(adj);

	adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON(sharp_x));
	int srp_x =  gtk_adjustment_get_value(adj);

	if (! (srp_x % 2) ) { // must be odd
		zippy_says("Sharpen X must be odd.");
		return;
		}

	adj = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON(sharp_y));
	int srp_y =  gtk_adjustment_get_value(adj);

	if (! (srp_y % 2) ) { // must be odd
		zippy_says("Sharpen Y must be odd.");
		return;
		}

	for (i = 0; i < (SP->scount); i++) {

		if (programData[i]->play) {

			programData[i]->play = 0;

			pcount++;

			new_name(filename, ProgramArray[i], "sharp");

                        sprintf(tmp, "ffmpeg -y -loglevel info -i \"%s\" "
                                "-vf fps=25,\"unsharp=luma_msize_x=%d:luma_msize_y=%d:luma_amount=%0.2f\"  -c:a libopus -b:a 128k -c:v libx264 -preset veryslow -crf 20  \"%s\"",
                                ProgramArray[i], srp_x, srp_y, luma, filename);

/*********************************** cuda trial ****************************************************
			sprintf(tmp, "ffmpeg -y -loglevel info -vsync 0 -hwaccel cuda  -i \"%s\" "
				"-vf \"unsharp=luma_msize_x=7:luma_msize_y=7:luma_amount=2.5\" "
				"-c:v h264_nvenc -b:v 100M \"%s\"",
                		ProgramArray[i], filename);
***************************************************************************************************/

			int err_stat = ff_run("Sharp", tmp, i);

		if (err_stat != 0) {
			zippy_says("An error occurred\n");
			sprintf(tmp, "rm \"%s\"", filename);
			return;
			}

			printf("Created: %s\n\n", filename);

			char tmp1[16000];
			sprintf(tmp1, "Created: %s\n", filename);
			zippy_says(tmp1);

			addToTop(filename);

			}
		}


	if (!pcount) {
		zippy_says("No files selected.\n");
		}

	clear_play();
	}

void	on_sharpen_pressed(GtkWidget *w) {
	}
